@@ -27,6 +27,7 @@ import os.path
 import logging
 import string
 import datetime
+import zlib
 
 
 from datetime import datetime
@@ -107,23 +108,28 @@ class AptHistory(object):
         except IOError, ioe:
             LOG.debug(ioe)
             return
-        for stanza in deb822.Deb822.iter_paragraphs(f):
-            # keep the UI alive
-            while self.main_context.pending():
-                self.main_context.iteration()
-            # ignore records with 
-            try:
-                trans = Transaction(stanza)
-            except KeyError, e:
-                continue
-            # ignore the ones we have already
-            if (rescan and
-                len(self.transactions) > 0 and
-                trans.start_date < self.transactions[0].start_date):
-                break
-            # add it
-            self.transactions.insert(0, trans)
-            
+        try:
+            for stanza in deb822.Deb822.iter_paragraphs(f):
+                # keep the UI alive
+                while self.main_context.pending():
+                    self.main_context.iteration()
+                # ignore records with 
+                try:
+                    trans = Transaction(stanza)
+                except KeyError, e:
+                    continue
+                # ignore the ones we have already
+                if (rescan and
+                    len(self.transactions) > 0 and
+                    trans.start_date < self.transactions[0].start_date):
+                    break
+                # add it
+                self.transactions.insert(0, trans)
+        except zlib.error, ze:
+            # corrupted history file, skip it
+            # (see http://bugs.launchpad.net/bugs/725186)
+            LOG.debug(ze)
+
     def _on_apt_history_changed(self, monitor, afile, other_file, event):
         if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
             self._scan(self.history_file, rescan = True)
