@@ -239,6 +239,15 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
     @inline_callbacks
     def reload(self, sources_list=None, metadata=None):
         """ reload package list """
+        # check if the sourcespart is there, if not, do a full reload
+        # this can happen when the "partner" repository is added, it
+        # will be in the main sources.list already and this means that
+        # aptsources will just enable it instead of adding a extra 
+        # sources.list.d file (LP: #666956)
+        d = apt_pkg.config.find_dir("Dir::Etc::sourceparts")
+        if (not sources_list or
+            not os.path.exists(os.path.join(d, sources_list))):
+            sources_list=""
         try:
             trans = yield self.aptd_client.update_cache(
                 sources_list=sources_list, defer=True)
@@ -276,7 +285,7 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
 
     @inline_callbacks
     def add_vendor_key_from_keyserver(self, keyid, 
-                                      keyserver="keyserver.ubuntu.com",
+                                      keyserver="hkp://keyserver.ubuntu.com:80/",
                                       metadata=None):
         # strip the keysize
         if "/" in keyid:
@@ -545,11 +554,14 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
         if trans.error_code == enums.ERROR_DAEMON_DIED:
             self._logger.warn("daemon dies, ignoring: %s %s" % (trans, enum))
             return
+        # hide any private ppa details in the error message since it may
+        # appear in the logs for LP bugs and potentially in screenshots as well
+        cleaned_error_details = obfuscate_private_ppa_details(trans.error_details)
         msg = "%s: %s\n%s\n\n%s" % (
             _("Error"),
             enums.get_error_string_from_enum(trans.error_code),
             enums.get_error_description_from_enum(trans.error_code),
-            trans.error_details)
+            cleaned_error_details)
         self._logger.error("error in _on_trans_finished '%s'" % msg)
         # show dialog to the user and exit (no need to reopen
         # the cache)
@@ -557,7 +569,7 @@ class AptdaemonBackend(gobject.GObject, TransactionsWatcher):
             None, 
             enums.get_error_string_from_enum(trans.error_code),
             enums.get_error_description_from_enum(trans.error_code),
-            trans.error_details)
+            cleaned_error_details)
 
     def _on_trans_finished(self, trans, enum):
         """callback when a aptdaemon transaction finished"""
