@@ -105,6 +105,7 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
                 'Purchased-Date' : 'purchase_date',
                 'PPA'        : 'archive_id',
                 'Icon'       : 'icon',
+                'Icon-Url'   : 'icon_url',
                 'Screenshot-Url' : 'screenshot_url',
                 'Thumbnail-Url' : 'thumbnail_url',
               }
@@ -118,11 +119,12 @@ class SoftwareCenterAgentParser(AppInfoParserBase):
         self.origin = "software-center-agent"
         self._apply_exceptions()
     def _apply_exceptions(self):
-        # map screenshot to thumbnail
+        # for items from the agent, we use the full-size screenshot for
+        # the thumbnail and scale it for display, this is done because
+        # we no longer keep thumbnail versions of screenshots on the server
         if (hasattr(self.sca_entry, "screenshot_url") and 
             not hasattr(self.sca_entry, "thumbnail_url")):
-            url = self.sca_entry.screenshot_url.replace(".png", ".thumb.png")
-            self.sca_entry.thumbnail_url = url
+            self.sca_entry.thumbnail_url = self.sca_entry.screenshot_url
         if hasattr(self.sca_entry, "description"):
             self.sca_entry.Comment = self.sca_entry.description.split("\n")[0]
             self.sca_entry.Description = "\n".join(self.sca_entry.description.split("\n")[1:])
@@ -386,12 +388,6 @@ def update_from_software_center_agent(db, cache):
         try:
             # magic channel
             entry.channel = AVAILABLE_FOR_PURCHASE_MAGIC_CHANNEL_NAME
-            # icon is transmited inline
-            iconname = "sc-agent-%s" % entry.package_name
-            icondata = base64.b64decode(entry.icon_data)
-            open(os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR,
-                              "%s.png" % iconname),"w").write(icondata)
-            entry.icon = iconname
             # now the normal parser
             parser = SoftwareCenterAgentParser(entry)
             index_app_info_from_parser(parser, db, cache)
@@ -405,7 +401,10 @@ def index_app_info_from_parser(parser, db, cache):
         doc = xapian.Document()
         term_generator.set_document(doc)
         # app name is the data
-        name = parser.get_desktop("Name")
+        if parser.has_option_desktop("X-GNOME-FullName"):
+            name = parser.get_desktop("X-GNOME-FullName")
+        else:
+            name = parser.get_desktop("Name")
         if name in seen:
             LOG.debug("duplicated name '%s' (%s)" % (name, parser.desktopf))
         seen.add(name)
@@ -423,9 +422,14 @@ def index_app_info_from_parser(parser, db, cache):
         doc.add_value(XAPIAN_VALUE_PKGNAME, pkgname)
         doc.add_value(XAPIAN_VALUE_DESKTOP_FILE, parser.desktopf)
         # cataloged_times
-        if pkgname in cataloged_times and "catalogedtime" in axi_values:
-            doc.add_value(axi_values["catalogedtime"], 
-                          xapian.sortable_serialise(cataloged_times[pkgname]))
+        if "catalogedtime" in axi_values:
+            if pkgname in cataloged_times:
+                doc.add_value(axi_values["catalogedtime"], 
+                              xapian.sortable_serialise(cataloged_times[pkgname]))
+            else:
+                # also catalog apps not found in axi (e.g. for-purchase apps)
+                doc.add_value(axi_values["catalogedtime"], 
+                              xapian.sortable_serialise(time.time()))
         # pocket (main, restricted, ...)
         if parser.has_option_desktop("X-AppInstall-Section"):
             archive_section = parser.get_desktop("X-AppInstall-Section")
@@ -456,6 +460,15 @@ def index_app_info_from_parser(parser, db, cache):
         if parser.has_option_desktop("X-AppInstall-PPA"):
             archive_ppa = parser.get_desktop("X-AppInstall-PPA")
             doc.add_value(XAPIAN_VALUE_ARCHIVE_PPA, archive_ppa)
+        # icon (for third party)
+        if parser.has_option_desktop("X-AppInstall-Icon-Url"):
+            doc.add_value(XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD, "1")
+            url = parser.get_desktop("X-AppInstall-Icon-Url")
+            doc.add_value(XAPIAN_VALUE_ICON_URL, url)
+            if not parser.has_option_desktop("X-AppInstall-Icon"):
+                # prefix pkgname to avoid name clashes
+                doc.add_value(XAPIAN_VALUE_ICON, "%s-icon-%s" % (
+                        pkgname, os.path.basename(url)))
         # screenshot (for third party)
         if parser.has_option_desktop("X-AppInstall-Screenshot-Url"):
             url = parser.get_desktop("X-AppInstall-Screenshot-Url")
