@@ -22,6 +22,8 @@ import xdg.BaseDirectory
 # buy-something base url
 #BUY_SOMETHING_HOST = "http://localhost:8000/"
 BUY_SOMETHING_HOST = os.environ.get("SOFTWARE_CENTER_BUY_HOST") or "https://software-center.ubuntu.com"
+BUY_SOMETHING_HOST_ANONYMOUS = os.environ.get("SOFTWARE_CENTER_BUY_HOST") or "http://software-center.ubuntu.com"
+
 
 # xapian pathes
 XAPIAN_BASE_PATH = "/var/cache/software-center"
@@ -86,6 +88,7 @@ XAPIAN_VALUE_SCREENSHOT_URL = 185
 XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD = 186
 XAPIAN_VALUE_THUMBNAIL_URL = 187
 XAPIAN_VALUE_SC_DESCRIPTION = 188
+XAPIAN_VALUE_ICON_URL = 190
 
 # fake channels
 PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME = "for-pay-needs-reinstall"
