@@ -29,6 +29,7 @@ import urllib
 import time
 import xml.sax.saxutils
 import gtk
+import dbus
 
 from enums import USER_AGENT, IMAGE_LOADING_INSTALLED
 
@@ -36,6 +37,8 @@ from enums import USER_AGENT, IMAGE_LOADING_INSTALLED
 # because only '&amp;', '&lt;', and '&gt;' are included by default
 ESCAPE_ENTITIES = {"&apos;":"'",
                    '&quot;':'"'}
+                   
+LOG = logging.getLogger("softwarecenter.utils")
 
 class ExecutionTime(object):
     """
@@ -172,6 +175,22 @@ def sources_filename_from_ppa_entry(entry):
     import apt_pkg
     name = "%s.list" % apt_pkg.URItoFileName(entry.uri)
     return name
+    
+def obfuscate_private_ppa_details(text):
+    """
+    hides any private PPA details that may be found in the given text
+    """
+    result = text
+    s = text.split()
+    for item in s:
+        if "private-ppa.launchpad.net" in item:
+            from urlparse import urlsplit
+            url_parts = urlsplit(item)
+            if url_parts.username:
+                result = result.replace(url_parts.username, "hidden")
+            if url_parts.password:
+                result = result.replace(url_parts.password, "hidden")
+    return result
 
 def release_filename_in_lists_from_deb_line(debline):
     """
@@ -184,6 +203,27 @@ def release_filename_in_lists_from_deb_line(debline):
                                     entry.dist)
     return name
     
+def get_default_language():
+    import locale
+    locale = locale.getdefaultlocale()
+    if not locale:
+        return "en"
+    if locale[0] == "C":
+        return "en"
+    return locale[0]
+    
+def is_unity_running():
+    """
+    return True if Unity is currently running
+    """
+    unity_running = False
+    try:
+        bus = dbus.SessionBus()
+        unity_running = bus.name_has_owner("com.canonical.Unity")
+    except:
+        LOG.exception("could not check for Unity dbus service")
+    return unity_running
+
 # FIXME: why not call it a generic downloader?
 class ImageDownloader(gobject.GObject):
 
