@@ -39,7 +39,7 @@ from softwarecenter.backend import get_install_backend
 from softwarecenter.db.application import AppDetails, Application
 from softwarecenter.enums import *
 from softwarecenter.paths import SOFTWARE_CENTER_ICON_CACHE_DIR
-from softwarecenter.utils import ImageDownloader, GMenuSearcher
+from softwarecenter.utils import ImageDownloader, GMenuSearcher, is_unity_running
 from softwarecenter.gwibber_helper import GWIBBER_SERVICE_AVAILABLE
 
 from appdetailsview import AppDetailsViewBase
@@ -519,6 +519,8 @@ class ScreenshotView(gtk.Alignment):
         and eventually the screenshot itself.
     """
 
+    MAX_THUMBNAIL_SIZE = 160, 160
+
     def __init__(self, distro, icons):
         # center child widgets in the available horizontal space (0.5)
         # 0.0 == left/top margin, 0.5 == center, 1.0 == right/bottom margin
@@ -683,6 +685,10 @@ class ScreenshotView(gtk.Alignment):
                 LOG.warn('Screenshot downloaded but the file could not be opened.')
                 return False
 
+            # if the thumbnail pixbuf is larger than 160 pixels wide, scale it
+            # to fit (see LP: #791932)
+            if pb.get_width() > 160:
+                pb = self._downsize_pixbuf(pb, *self.MAX_THUMBNAIL_SIZE)
             self.image.set_size_request(-1, -1)
             self.image.set_from_pixbuf(pb)
             # start the fade in
@@ -693,6 +699,20 @@ class ScreenshotView(gtk.Alignment):
         gobject.idle_add(setter_cb, screenshot_path)
         return
 
+    def _downsize_pixbuf(self, pb, target_w, target_h):
+        w = pb.get_width()
+        h = pb.get_height()
+
+        if w > h:
+            sf = float(target_w) / w
+        else:
+            sf = float(target_h) / h
+
+        sw = int(w*sf)
+        sh = int(h*sf)
+
+        return pb.scale_simple(sw, sh, gtk.gdk.INTERP_BILINEAR)
+
     def get_is_actionable(self):
         """ Returns true if there is a screenshot available and the download has completed """
         return self.screenshot_available and self.ready
@@ -1409,6 +1429,9 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         return
 
     def _configure_where_is_it(self):
+        # disable where-is-it under Unity as it does not apply there
+        if is_unity_running():
+            return
         # remove old content
         self.desc_installed_where.foreach(lambda c: c.destroy())
         self.desc_installed_where.set_property("can-focus", False)
