@@ -96,6 +96,13 @@ class AppStore(gtk.GenericTreeModel):
 
     # the default result size for a search
     DEFAULT_SEARCH_LIMIT = 200
+    
+    __gsignals__ = {
+        "needs-refresh" : (gobject.SIGNAL_RUN_LAST,
+                           None, 
+                           (str, ),
+                           ),
+        }
 
     def __init__(self, cache, db, icons, search_query=None, 
                  limit=DEFAULT_SEARCH_LIMIT,
@@ -440,7 +447,7 @@ class AppStore(gtk.GenericTreeModel):
         self._installable_apps = None
 
 
-    def _download_icon_and_show_when_ready(self, cache, pkgname, icon_file_name):
+    def _download_icon_and_show_when_ready(self, cache, pkgname, icon_file_name, icon_url):
         self._logger.debug("did not find the icon locally, must download %s" % icon_file_name)
         def on_image_download_complete(downloader, image_file_path):
             pb = gtk.gdk.pixbuf_new_from_file_at_size(icon_file_path,
@@ -449,8 +456,15 @@ class AppStore(gtk.GenericTreeModel):
             # replace the icon in the icon_cache now that we've got the real one
             icon_file = os.path.splitext(os.path.basename(image_file_path))[0]
             self.icon_cache[icon_file] = pb
+            self.emit("needs-refresh", pkgname)
         
-        url = get_distro().get_downloadable_icon_url(cache, pkgname, icon_file_name)
+        if icon_url:
+            url = icon_url
+        else:
+            # if the icon url has not been provided by the agent, then this
+            # downloadable icon is from the extras repository and so we must
+            # generate the download url
+            url = get_distro().get_downloadable_icon_url(cache, pkgname, icon_file_name)
         icon_file_path = os.path.join(SOFTWARE_CENTER_ICON_CACHE_DIR, icon_file_name)
         image_downloader = ImageDownloader()
         image_downloader.connect('image-download-complete', on_image_download_complete)
@@ -565,9 +579,13 @@ class AppStore(gtk.GenericTreeModel):
                             self.icon_cache[icon_name] = icon
                             return icon
                     elif self.db.get_icon_needs_download(doc):
+                        # check if this is a downloadable icon for an
+                        # application provided by the agent
+                        icon_url = self.db.get_icon_url(doc)
                         self._download_icon_and_show_when_ready(self.cache, 
                                                                 app.pkgname,
-                                                                icon_file_name)
+                                                                icon_file_name,
+                                                                icon_url)
                         # display the missing icon while the real one downloads
                         self.icon_cache[icon_name] = self._appicon_missing_icon
             except glib.GError, e:
@@ -1069,7 +1087,11 @@ class CellRendererAppView2(gtk.CellRendererText):
 
         # important! ensures correct text rendering, esp. when using hicolor theme
         if (flags & gtk.CELL_RENDERER_SELECTED) != 0:
-            state = gtk.STATE_SELECTED
+            # this follows the behaviour that gtk+ uses for states in treeviews
+            if widget.has_focus():
+                state = gtk.STATE_SELECTED
+            else:
+                state = gtk.STATE_ACTIVE
         else:
             state = gtk.STATE_NORMAL
 
@@ -1567,10 +1589,13 @@ class AppView(gtk.TreeView):
             action_btn.set_sensitive(True)
             self._set_cursor(action_btn, self._cursor_hand)
 
-    def _on_transaction_stopped(self, backend, pkgname, tr):
+    def _on_transaction_stopped(self, backend, result, tr):
         """ callback when an application install/remove transaction has stopped """
         # remove pkg from the block list
-        self._check_remove_pkg_from_blocklist(pkgname)
+        if isinstance(result, str):
+            self._check_remove_pkg_from_blocklist(result)
+        else:
+            self._check_remove_pkg_from_blocklist(result.pkgname)
 
         action_btn = tr.get_button_by_name('action0')
         if action_btn:
