@@ -1,7 +1,8 @@
 # Copyright (C) 2010 Canonical
 #
 # Authors:
-#  Michael Vogt, Gary Lasker
+#  Michael Vogt
+#  Gary Lasker
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -27,13 +28,16 @@ import gobject
 
 from gettext import gettext as _
 
-from softwarecenter.enums import *
+from softwarecenter.backend import get_install_backend
 from softwarecenter.distro import get_distro
+from softwarecenter.enums import *
 
 from appview import AppView, AppStore, AppViewFilter
 
 from softwarepane import SoftwarePane, wait_for_apt_cache_ready
 
+LOG = logging.getLogger(__name__)
+
 class ChannelPane(SoftwarePane):
     """Widget that represents the channel pane for display of
        individual channels (PPAs, partner repositories, etc.)
@@ -44,17 +48,20 @@ class ChannelPane(SoftwarePane):
     (PAGE_APPLIST,
      PAGE_APP_DETAILS) = range(2)
 
-    def __init__(self, cache, db, distro, icons, datadir):
+    def __init__(self, cache, history, db, distro, icons, datadir):
         # parent
-        SoftwarePane.__init__(self, cache, db, distro, icons, datadir, show_ratings=False)
+        SoftwarePane.__init__(self, cache, history, db, distro, icons, datadir,
+                              show_ratings=False)
         self.channel = None
         self.apps_filter = None
         self.search_terms = ""
         self.current_appview_selection = None
+        self.refresh_handler_id = None
         self.distro = get_distro()
         # UI
         self._build_ui()
-        
+        self.connect("app-list-changed", self._on_app_list_changed)
+
     def _build_ui(self):
         self.notebook.append_page(self.scroll_app_list, gtk.Label("channel"))
         # details
@@ -76,6 +83,8 @@ class ChannelPane(SoftwarePane):
         """refresh the applist after search changes and update the 
            navigation bar
         """
+        LOG.debug("refresh_apps")
+        self.show_appview_spinner()
         if not self.channel:
             return
         self.refresh_seq_nr += 1
@@ -85,22 +94,41 @@ class ChannelPane(SoftwarePane):
                                                              channel_query)
             self.navigation_bar.add_with_id(_("Search Results"),
                                             self.on_navigation_search, 
-                                            "search")
+                                            "search", do_callback=False)
         else:
-            self.navigation_bar.remove_all(keep_first_part=False)
-            self.navigation_bar.add_with_id(self.channel.get_channel_display_name(),
-                                        self.on_navigation_list,
-                                        "list")
+            self.navigation_bar.add_with_id(
+                self.channel.get_channel_display_name(),
+                self.on_navigation_list,
+                "list", do_callback=False)
             query = xapian.Query(channel_query)
 
-        logging.debug("channelpane query: %s" % query)
-        # deactivate the old model, otherwise we have a memleak and
-        # a cpu leak
-        self.app_view.clear_model()
-        gobject.idle_add(self._make_new_model, query, self.refresh_seq_nr)
+        LOG.debug("channelpane query: %s" % query)
+        # *ugh* deactivate the old model because otherwise it keeps
+        # getting progress_changed events and eats CPU time until its
+        # garbage collected
+        old_model = self.app_view.get_model()
+        
+        # if the list is expected to contain many items, 
+        #  clear the current model to display
+        # an empty list while the full list is generated; 
+        #  this prevents a visual glitch when
+        # the list is replaced
+        if ((self.channel.get_channel_name() == self.distro.get_distro_channel_name() and not 
+             self.search_terms)):
+            self.app_view.clear_model()
+        
+        if old_model is not None:
+            old_model.active = False
+            while gtk.events_pending():
+                gtk.main_iteration()
+        self._make_new_model(query, self.refresh_seq_nr)
         return False
 
     def _make_new_model(self, query, seq_nr):
+        # something changed already
+        if self.refresh_seq_nr != seq_nr:
+            LOG.warn("early discarding new model (%s != %s)" % (seq_nr, self.refresh_seq_nr))
+            return False
         # get a new store and attach it to the view
         if self.scroll_app_list.window:
             self.scroll_app_list.window.set_cursor(self.busy_cursor)
@@ -109,18 +137,33 @@ class ChannelPane(SoftwarePane):
                              self.icons, 
                              query, 
                              limit=0,
-                             sort=True,
+                             sortmode=self.channel.get_channel_sort_mode(),
+                             nonapps_visible = self.nonapps_visible,
                              filter=self.apps_filter)
         # between request of the new model and actual delivery other
         # events may have happend
+        self.hide_appview_spinner()
         if self.scroll_app_list.window:
             self.scroll_app_list.window.set_cursor(None)
         if seq_nr == self.refresh_seq_nr:
+            if (self.refresh_handler_id and
+                self.app_view.handler_is_connected(self.refresh_handler_id)):
+                self.app_view.disconnect(self.refresh_handler_id)
             self.app_view.set_model(new_model)
-            self.emit("app-list-changed", len(new_model))
+            self.app_view.get_model().active = True
+            # connect the refresh signal from the model
+            self.refresh_handler_id = self.app_view.get_model().connect(
+                                                        "needs-refresh",
+                                                        self._on_needs_refresh)
+            # we can not use "new_model" here, because set_model may actually
+            # discard new_model and just update the previous one
+            self.emit("app-list-changed", len(self.app_view.get_model()))
         else:
-            logging.debug("discarding new model (%s != %s)" % (seq_nr, self.refresh_seq_nr))
+            LOG.debug("discarding new model (%s != %s)" % (seq_nr, self.refresh_seq_nr))
         return False
+        
+    def _on_needs_refresh(self, appstore, pkgname):
+        self.app_view.queue_draw()
 
     def set_channel(self, channel):
         """
@@ -128,17 +171,55 @@ class ChannelPane(SoftwarePane):
         and set up the AppViewFilter if required
         """
         self.channel = channel
-        if self.channel.filter_required:
+        # check to see if there is any section info that needs to be applied
+        if channel._channel_color:
+            self.section.set_color(channel._channel_color)
+        if channel._channel_image_id:
+            self.section.set_image_id(channel._channel_image_id)
+        self.section_sync()
+
+        # check if the channel needs to added
+        if channel.needs_adding and channel._source_entry:
+            dialog = gtk.MessageDialog(flags=gtk.DIALOG_MODAL,
+                                       type=gtk.MESSAGE_QUESTION)
+            dialog.set_title("")
+            dialog.set_markup("<big><b>%s</b></big>" % _("Add channel"))
+            dialog.format_secondary_text(_("The selected channel is not yet "
+                                           "added. Do you want to add it now?"))
+            dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                               gtk.STOCK_ADD, gtk.RESPONSE_YES)
+            res = dialog.run()
+            dialog.destroy()
+            if res == gtk.RESPONSE_YES:
+                channel.needs_adding = False
+                backend = get_install_backend()
+                backend.add_sources_list_entry(channel._source_entry)
+                backend.emit("channels-changed", True)
+                backend.reload()
+            return
+        # normal operation
+        # always show all packages in the partner repository
+        # FIXME:  remove this special case code in favor of a more general solution
+        if channel.get_channel_component() == "partner":
+            self.nonapps_visible = True
+            self.disable_show_hide_nonapps = True
+        else:
+            self.nonapps_visible = False
+            self.disable_show_hide_nonapps = False
+        self.apps_filter = None
+        if self.channel.only_packages_without_applications:
             self.apps_filter = AppViewFilter(self.db, self.cache)
             self.apps_filter.set_only_packages_without_applications(True)
-        else:
-            self.apps_filter = None
-        # when displaying a new channel, clear any search in progress
-        self.search_terms = ""
+        if self.channel.installed_only:
+            if self.apps_filter is None:
+                self.apps_filter = AppViewFilter(self.db, self.cache)
+            self.apps_filter.set_installed_only(True)
+        # switch to applist, this will clear searches too
+        self.display_list()
         
     def on_search_terms_changed(self, searchentry, terms):
         """callback when the search entry widget changes"""
-        logging.debug("on_search_terms_changed: '%s'" % terms)
+        LOG.debug("on_search_terms_changed: '%s'" % terms)
         self.search_terms = terms
         if not self.search_terms:
             self._clear_search()
@@ -146,8 +227,9 @@ class ChannelPane(SoftwarePane):
         self.notebook.set_current_page(self.PAGE_APPLIST)
         
     def on_db_reopen(self, db):
+        LOG.debug("got db-reopen signal")
         self.refresh_apps()
-        self._show_channel_overview()
+        self.app_details.refresh_app()
 
     def on_navigation_search(self, button, part):
         """ callback when the navigation button with id 'search' is clicked"""
@@ -157,6 +239,9 @@ class ChannelPane(SoftwarePane):
         """callback when the navigation button with id 'list' is clicked"""
         if not button.get_active():
             return
+        self.display_list()
+    
+    def display_list(self):
         self._clear_search()
         self._show_channel_overview()
         # only emit something if the model is there
@@ -168,14 +253,24 @@ class ChannelPane(SoftwarePane):
         """callback when the navigation button with id 'details' is clicked"""
         if not button.get_active():
             return
+        self.display_details()
+    
+    def display_details(self):
         self.notebook.set_current_page(self.PAGE_APP_DETAILS)
         self.searchentry.hide()
+        self.action_bar.clear()
         
     def on_application_selected(self, appview, app):
         """callback when an app is selected"""
-        logging.debug("on_application_selected: '%s'" % app)
+        LOG.debug("on_application_selected: '%s'" % app)
         self.current_appview_selection = app
 
+    def _on_app_list_changed(self, pane, length):
+        """internal helper that keeps the the action bar up-to-date by
+           keeping track of the app-list-changed signals
+        """
+        self.update_show_hide_nonapps()
+
     def display_search(self):
         self.navigation_bar.remove_id("details")
         self.notebook.set_current_page(self.PAGE_APPLIST)
@@ -195,14 +290,24 @@ class ChannelPane(SoftwarePane):
         if not model:
             return ""
         length = len(self.app_view.get_model())
-        if len(self.searchentry.get_text()) > 0:
-            return gettext.ngettext("%s matching item",
-                                    "%s matching items",
-                                    length) % length
+        if self.channel.installed_only:
+            if len(self.searchentry.get_text()) > 0:
+                return gettext.ngettext("%(amount)s matching item",
+                                        "%(amount)s matching items",
+                                        length) % { 'amount' : length, }
+            else:
+                return gettext.ngettext("%(amount)s item installed",
+                                        "%(amount)s items installed",
+                                        length) % { 'amount' : length, }
         else:
-            return gettext.ngettext("%s item available",
-                                    "%s items available",
-                                    length) % length
+            if len(self.searchentry.get_text()) > 0:
+                return gettext.ngettext("%(amount)s matching item",
+                                        "%(amount)s matching items",
+                                        length) % { 'amount' : length, }
+            else:
+                return gettext.ngettext("%(amount)s item available",
+                                        "%(amount)s items available",
+                                        length) % { 'amount' : length, }
                                     
     def get_current_app(self):
         """return the current active application object applicable
@@ -212,6 +317,10 @@ class ChannelPane(SoftwarePane):
     def is_category_view_showing(self):
         # there is no category view in the channel pane
         return False
+        
+    def is_applist_view_showing(self):
+        """Return True if we are in the applist view """
+        return self.notebook.get_current_page() == self.PAGE_APPLIST
 
 if __name__ == "__main__":
     #logging.basicConfig(level=logging.DEBUG)
@@ -228,7 +337,7 @@ if __name__ == "__main__":
     db = xapian.Database(pathname)
     icons = gtk.icon_theme_get_default()
     icons.append_search_path("/usr/share/app-install/icons/")
-    cache = apt.Cache(apt.progress.OpTextProgress())
+    cache = apt.Cache(apt.progress.text.OpProgress())
     cache.ready = True
 
     w = ChannelPane(cache, db, icons, datadir)
