@@ -19,7 +19,33 @@
 
 import gtk
 from gettext import gettext as _
-from pkgview import PkgNamesView
+
+class SimpleGtkbuilderDialog(object):
+    def __init__(self, datadir, domain):
+        # setup ui
+        self.builder = gtk.Builder()
+        self.builder.set_translation_domain(domain)
+        self.builder.add_from_file(datadir+"/ui/dialogs.ui")
+        self.builder.connect_signals(self)
+        for o in self.builder.get_objects():
+            if issubclass(type(o), gtk.Buildable):
+                name = gtk.Buildable.get_name(o)
+                setattr(self, name, o)
+            else:
+                print >> sys.stderr, "WARNING: can not get name for '%s'" % o
+
+
+def confirm_repair_broken_cache(parent, datadir):
+    glade_dialog = SimpleGtkbuilderDialog(datadir, domain="software-center")
+    dialog = glade_dialog.dialog_broken_cache
+    dialog.set_default_size(380, -1)
+    dialog.set_transient_for(parent)
+
+    result = dialog.run()
+    dialog.destroy()
+    if result == gtk.RESPONSE_ACCEPT:
+        return True
+    return False
 
 class DetailsMessageDialog(gtk.MessageDialog):
     """Message dialog with optional details expander"""
@@ -74,44 +100,17 @@ def error(parent, primary, secondary, details=None):
                          details=details,
                          type=gtk.MESSAGE_ERROR)
 
-def confirm_remove(parent, primary, cache, button_text, icon_path, depends=None):
-    """Confirm removing of the given app with the given depends"""
-    dialog = gtk.MessageDialog(parent=parent, flags=0, 
-                               type=gtk.MESSAGE_QUESTION, 
-                               message_format=None)
-    dialog.set_resizable(True)
-    dialog.add_button(_("Cancel"), gtk.RESPONSE_CANCEL)
-    dialog.add_button(button_text, gtk.RESPONSE_ACCEPT)
-
-    # fixes launchpad bug #560021
-    pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(icon_path, 32, 32)
-    image = dialog.get_image()
-    image.set_from_pixbuf(pixbuf)
-
-    dialog.set_markup(primary)
-    # add the dependencies
-    if depends:
-        vbox = dialog.get_content_area()
-        # FIXME: make this a generic pkgview widget
-        view = PkgNamesView(_("Dependency"), cache, depends)
-        view.set_headers_visible(False)
-        scrolled = gtk.ScrolledWindow()
-        scrolled.set_size_request(-1, 200)
-        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
-        scrolled.add(view)
-        scrolled.show_all()
-        # FIXME: this needs padding on the left side so 
-        # it lines up with the text
-        vbox.pack_start(scrolled)
-    result = dialog.run()
-    dialog.hide()
-    if result == gtk.RESPONSE_ACCEPT:
-        return True
-    return False
-    
 
 if __name__ == "__main__":
+    print "Running remove dialog"
+    
+                   
+    print "Running broken apt-cache dialog"               
+    confirm_repair_broken_cache(None, "./data")
+                   
+    print "Showing message dialog"
     messagedialog(None, primary="first, no second")
+    print "showing error"
     error(None, "first", "second")
     error(None, "first", "second", "details ......")
-    
+     
